"""
MusicChecker is a class aimed at scanning a series of media files, extracting tag information from them for later reporting
"""

VERSION = (0, 1, 1)
"""Version tuple."""

VERSION_STRING = ".".join(map(str, VERSION))
"""Version string."""
